package com.qianhe.system.service.impl;

import com.qianhe.common.utils.DateUtils;
import com.qianhe.common.utils.StringUtils;
import com.qianhe.system.domain.WaterGoods;
import com.qianhe.system.domain.WaterOrder;
import com.qianhe.system.domain.WaterOrderGoods;
import com.qianhe.system.mapper.WaterGoodsMapper;
import com.qianhe.system.mapper.WaterOrderGoodsMapper;
import com.qianhe.system.mapper.WaterOrderMapper;
import com.qianhe.system.service.IWaterOrderService;
import com.qianhe.system.vo.WaterOrderVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Random;

/**
 * 订单Service业务层处理
 *
 * @author qianhe
 * @date 2023-11-23
 */
@Service
public class WaterOrderServiceImpl implements IWaterOrderService
{
    @Autowired
    private WaterOrderMapper waterOrderMapper;
    @Autowired
    private WaterOrderGoodsMapper waterOrderGoodsMapper;
    @Autowired
    private WaterGoodsMapper waterGoodsMapper;

    /**
     * 查询订单
     *
     * @param id 订单主键
     * @return 订单
     */
    @Override
    public WaterOrderVo selectWaterOrderById(Long id)
    {
        WaterOrder waterOrder = waterOrderMapper.selectWaterOrderById(id);
        //转vo
        WaterOrderVo waterOrderVo = new WaterOrderVo();
        BeanUtils.copyProperties(waterOrder,waterOrderVo);
        //查询该订单商品
        WaterOrderGoods waterOrderGoods = new WaterOrderGoods();
        waterOrderGoods.setOrderId(id);
        List<WaterOrderGoods> waterOrderGoodsList = waterOrderGoodsMapper.selectWaterOrderGoodsList(waterOrderGoods);
        if (waterOrderGoodsList.size() > 0){
            waterOrderVo.setWaterOrderGoodsList(waterOrderGoodsList);
        }
        return waterOrderVo;
    }

    /**
     * 查询订单列表
     *
     * @param waterOrderVo 订单
     * @return 订单
     */
    @Override
    public List<WaterOrder> selectWaterOrderList(WaterOrderVo waterOrderVo)
    {
        return waterOrderMapper.selectWaterOrderList(waterOrderVo);
    }

    /**
     * 新增订单
     *
     * @param waterOrderVo 订单
     * @return 结果
     */
    @Override
    @Transactional
    public int insertWaterOrder(WaterOrderVo waterOrderVo)
    {
        waterOrderVo.setCreateUser(waterOrderVo.getUserId().toString());
        waterOrderVo.setCreateTime(DateUtils.getNowDate());
        waterOrderVo.setOrderNum(getOrerNum());
        waterOrderVo.setOrderState(1);
        waterOrderVo.setPayState(0);
        waterOrderVo.setOrderType(1);
        int i = waterOrderMapper.insertWaterOrder(waterOrderVo);
        //新增订单商品
        insertWaterOrderGoods(waterOrderVo);
        return i;
    }

    /**
     * 生成订单编号
     * @return
     */
    private String getOrerNum(){
        String orderNum = "";
        //获取当前年月日
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        String today = format.format(DateUtils.getNowDate());
        Long rqbh = Long.parseLong(today + "0001");
        while (true){
            WaterOrder waterOrder = new WaterOrder();
            waterOrder.setOrderNum(rqbh.toString());
            WaterOrder waterOrder1 = waterOrderMapper.selectWaterOrderNumByOrderNum(rqbh.toString());
            if (StringUtils.isNotNull(waterOrder1)){
                rqbh ++;
            }else {
                break;
            }
        }
        //生成随机四位英文字母
        Random random = new Random();
        String randomletters = "";
        for (int i = 0; i < 4; i++) {
            int index = random.nextInt(26);
            char letter = (char) ('A' + index);
            randomletters += letter;
        }
        orderNum = rqbh + randomletters;
        return orderNum;
    }

    private void insertWaterOrderGoods(WaterOrderVo waterOrderVo) {
        List<WaterOrderGoods> waterOrderGoodsList = waterOrderVo.getWaterOrderGoodsList();
        //商品订单主键id
        Long id = waterOrderVo.getId();
        //商品订单编号
        String orderNum = waterOrderVo.getOrderNum();
        if (StringUtils.isNotNull(waterOrderGoodsList)){
            for (WaterOrderGoods waterOrderGoods : waterOrderGoodsList) {
                //根据商品id查询商品信息
                WaterGoods waterGoods = waterGoodsMapper.selectWaterGoodsById(waterOrderGoods.getGoodsId());
                waterOrderGoods.setOrderId(id);
                waterOrderGoods.setOrderNum(orderNum);
                waterOrderGoods.setGoodsTypeId(waterGoods.getGoodsTypeId());
                waterOrderGoods.setGoodsTitle(waterGoods.getTitle());
                waterOrderGoods.setCreateUser(waterOrderVo.getUserId().toString());
                waterOrderGoods.setCreateTime(DateUtils.getNowDate());
            }
            //批量新增订单商品
            waterOrderMapper.batchInsertWaterOrderGoods(waterOrderGoodsList);
        }
    }

    /**
     * 修改订单
     *
     * @param waterOrderVo 订单
     * @return 结果
     */
    @Override
    @Transactional
    public int updateWaterOrder(WaterOrderVo waterOrderVo)
    {
        //删除订单商品
        waterOrderMapper.deleteWaterOrderGoodsByOrderId(waterOrderVo.getId());
        //新增订单商品
        insertWaterOrderGoods(waterOrderVo);
        return waterOrderMapper.updateWaterOrder(waterOrderVo);
    }

    /**
     * 批量删除订单
     *
     * @param ids 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteWaterOrderByIds(Long[] ids)
    {
        //批量删除订单商品
        waterOrderMapper.deleteWaterOrderGoodsByOrderIds(ids);
        return waterOrderMapper.deleteWaterOrderByIds(ids);
    }

    /**
     * 删除订单信息
     *
     * @param id 订单主键
     * @return 结果
     */
    @Override
    public int deleteWaterOrderById(Long id)
    {
        return waterOrderMapper.deleteWaterOrderById(id);
    }

    /**
     * 修改订单状态
     * @param waterOrderVo
     * @return
     */
    @Override
    public int updateOrderState(WaterOrderVo waterOrderVo) {
        return waterOrderMapper.updateWaterOrder(waterOrderVo);
    }
}
